package dm4509.tcp;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.StandardSocketOptions;
import java.nio.channels.ServerSocketChannel;


/**
 * Classe qui crée un serveur en attende de connexion et qui fournit une méthode pour construire une instance de TcpSocket connectée à chaque client accepté.
 * @author Eric Lallet.
 *
 */
public class TcpServer  implements AutoCloseable {
	/**
	 * le canal d'écoute du serveur.
	 */
	private ServerSocketChannel listenChannel;
	
	
	/**
	 * Constructeur qui crée un serveur TCP en écoute sur le port passé en paramètre.
	 * @param port
	 *            numéro du port d'écoute.
	 * @throws IOException
	 *            toutes les exceptions d'entrées/sorties.
	 */
	public TcpServer(int port) throws IOException {
		listenChannel = ServerSocketChannel.open();
		
		listenChannel.setOption(StandardSocketOptions.SO_REUSEADDR, true);
		listenChannel.bind(new InetSocketAddress(port));
		
	}
	
	/**
	 * Accepte les nouveaux clients et crée une instance de TcpScoket à partir de cette nouvelle connexion.
	 * @return
	 *         la référence sur l'instance de TcpSocket créée.
	 * @throws IOException
	 *          toutes les exceptions d'entrées/sorties.
	 */
	public TcpSocket acceptClient() throws IOException {
		return new TcpSocket(listenChannel.accept());
		
		
	}

	@Override
	public void close() throws Exception {
		listenChannel.close();
		
	}

}
