package dm4509.tcpservices;

import static dm4509.common.Log.LOG_ON;
import static dm4509.common.Log.TEST;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InterruptedIOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;

import org.apache.log4j.Level;

import dm4509.tcp.*;

public class TcpTestTools {
	/**
	 * thread du service serveur.
	 */
	private Thread serverThread = null;

	/**
	 * thread du service client.
	 */
	private Thread clientThread = null;
	
	/**
	 * thread du service echo.
	 */
	private Thread echoThread = null;

	/**
	 * thread du service server send file.
	 */
	private Thread serverSendFileThread = null;

	/**
	 * thread du service server receive file.
	 */
	private Thread serverReceiveFileThread = null;


	/**
	 * thread du service client send file.
	 */
	private Thread clientSendFileThread = null;

	/**
	 * thread du service client receive file.
	 */
	private Thread clientReceiveFileThread = null;


	/**
	 * thread du service client receive file.
	 */
	private Thread alarmThread = null;


	/**
	 * Aucune initialisation à la constructions. Chaque service se charge d'initialiser ce dont il a besoin.
	 */
	public TcpTestTools() {
		
	}
	
	/**
	 * Interrompt un Thread lancé précédemment. 
	 * @param t
	 *         référence sur le thread à interrompre.
	 * @param message
	 *         message de service qui annonce la fin du thread dans le log niveau trace.
	 *         Passer une chaîne null pour rendre la méthode muette.
	 * @throws InterruptedException
	 *         exception en cas d'interruption pendant join().
	 */
	private static void serviceStop(Thread t, String message) throws InterruptedException {
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			if (message != null) {
				TEST.trace("TcpTestTools: " +  message);
			}
		}
		if (t != null) {
			t.interrupt();
			t.join();
			t = null;
		}
		
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			if (message != null) {
				TEST.trace("TcpTestTools: Thread terminé");
			}
		}
		
	}


	/**
	 * Crée un serveur TCP localhost:port qui attend la connexion des clients en boucle infinie 
	 * (utiliser  un appel à serverStop() pour arréter ce service).
	 * @param port
	 *            port du serveur TCP.
	 * @throws InterruptedException 
	 *             exception en cas d'interruption pendant l'attente de la fin du thread.
	 */
	public void serverStart(int port) throws InterruptedException {

		Runnable echoRun = new Runnable() {
			@Override
			public void run() {
				if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
					TEST.trace("TcpTestTools: Lancement du serveur sur le port " + port);
				}
				try (TcpServer echoServer = new TcpServer(port);){

					while (true) {
						echoServer.acceptClient();
					}

				} catch (InterruptedIOException e) {

					Thread.currentThread().interrupt();


				} catch (Exception e1) {
					if (e1.getMessage() != null) {
						if (LOG_ON && TEST.isEnabledFor(Level.DEBUG)) {
							TEST.debug("TcpTestTools:  Exception:" + e1.getMessage());
						}
					}
					// fin du thread sans autre explication... TODO traiter les fins anormales!
				}
				if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
					TEST.trace("TcpTestTools:  Fin du serveur de test");
				}
				serverThread = null;

			}
			

		};

		serverThread = new Thread(echoRun);
		serverThread.start();
		Thread.sleep(1000); // pour être sûr que le serveur est lancé
	}
	
	/**
	 * Interompt le thread qui réalise le service de lancement d'un serveur.
	 * @throws InterruptedException
	 *          exception en cas d'interruption pendant join().
	 */
	public void serverStop() throws InterruptedException {
		serviceStop(serverThread, "Interruption du service serveur");
	}



	/**
	 * Crée un client TCP qui se connecte à localhost:port.
	 * @param port
	 *            port du serveur TCP.
	 */
	public void clienStart(int port) {

		Runnable echoRun = new Runnable() {
			@Override
			public void run() {
				if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
					TEST.trace("TcpTestTools: Lancement d'un client se connectant au port " + port);
				}
				try (TcpSocket client = new TcpSocket("localhost", port); ) {


				}
				catch (InterruptedIOException e) {
					Thread.currentThread().interrupt();

				} catch (IOException e1) {
					if (e1.getMessage() != null) {
						if (e1.getMessage() != null) {
							if (LOG_ON && TEST.isEnabledFor(Level.DEBUG)) {
								TEST.debug("TcpTestTools:  Exception:" + e1.getMessage());
							}
						}
					}
					// fin du thread sans autre explication... TODO traiter les fins anormales!
				}

				if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
					TEST.trace("TcpTestTools: Fin du client de test");
				}
				clientThread = null;


			}

		};

		clientThread = new Thread(echoRun);
		clientThread.start();
	}



	/**
	 * Interompt le thread qui réalise le service de lancement d'un client.
	 * @throws InterruptedException
	 *          exception en cas d'interruption pendant join().
	 */
	public void clientStop() throws InterruptedException {
		serviceStop(clientThread, "Interruption du service client");
	}



	/**
	 * Crée un serveur TCP localhost:port qui renvoie en echo tous les octets qui lui sont envoyés par le premier client connecté.
	 * @param port
	 *            port du serveur TCP.
	 * @throws InterruptedException 
	 *           exception en cas d'interruption pendant l'attente de la fin du thread.
	 */
	public void echoStart(int port) throws InterruptedException {

		Runnable echoRun = new Runnable() {
			@Override
			public void run() {
				if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
					TEST.trace("TcpTestTools: Lancement du service echo");
				}
				try (TcpServer echoServer = new TcpServer(port);){

					TcpSocket connectedToClient = echoServer.acceptClient();
					connectedToClient.echo(); // boucle tant que le client est connecté

				} catch (InterruptedIOException e) {

					Thread.currentThread().interrupt();


				} catch (Exception e1) {
					if (e1.getMessage() != null) {
						if (e1.getMessage() != null) {
							if (LOG_ON && TEST.isEnabledFor(Level.DEBUG)) {
								TEST.debug("TcpTestTools:  Exception:" + e1.getMessage());
							}
						}
					}
					// fin du thread sans autre explication... TODO traiter les fins anormales!
				} 

				if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
					TEST.trace("TcpTestTools: Fin du service echo");
				}
				echoThread = null;

			}

		};

		echoThread = new Thread(echoRun);
		echoThread.start();
		Thread.sleep(1000); // pour être sûr que le serveur est lancé
	}

	/**
	 * Interrompt le thread qui réalise le service d'echo.
	 * @throws InterruptedException 
	 *           exception en cas d'interruption pendant join().
	 */
	public void echoStop() throws InterruptedException {
		serviceStop(echoThread, "Interruption du service echo");
	}

	/**
	 * Crée un serveur TCP qui envoie au premier client connecté le contenu d'un fichier.
	 * @param port
	 *            port du serveur TCP.
	 * @param fileName
	 *            nom du fichier à envoyer.
	 * @throws InterruptedException 
	 *          exception en cas d'interruption pendant l'attente de la fin du thread.
	 */
	public void serverSendFileStart(int port, String fileName) throws InterruptedException {
		Runnable serverSendFileRun = new Runnable() {
			@Override
			public void run() {
				try (TcpServer sendFileServer = new TcpServer(port);
						FileInputStream fin = new FileInputStream(fileName);){
					if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
						TEST.trace("TcpTestTools: Lancement du service serveur send file sur le port " + port);
					}

					TcpSocket connectedToClient = sendFileServer.acceptClient();

					FileChannel fcin = fin.getChannel();
					ByteBuffer buffer = ByteBuffer.allocate(1024);
					int lu; 
					do {
						buffer.clear();
						lu = fcin.read(buffer);
						buffer.flip();
						connectedToClient.sendBuffer(buffer);
					} while (lu == 1024);

					connectedToClient.close();
				} catch (InterruptedIOException e) {
					Thread.currentThread().interrupt();

				} catch (Exception e1) {
					if (e1.getMessage() != null) {
						if (e1.getMessage() != null) {
							if (LOG_ON && TEST.isEnabledFor(Level.DEBUG)) {
								TEST.debug("TcpTestTools:  Exception:" + e1.getMessage());
							}
						}
					}
					// fin du thread sans autre explication... TODO traiter les fins anormales!
				} 

				if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
					TEST.trace("TcpTestTools: Fin du service serveur send file");
				}
				serverSendFileThread = null;
				
			}

		};
		serverSendFileThread = new Thread(serverSendFileRun);
		serverSendFileThread.start();
		Thread.sleep(1000); // pour être sûr que le serveur est lancé
	}

	/**
	 * Interrompt le thread qui réalise le service server send file.
	 * @throws InterruptedException
	 *         exception en cas d'interruption pendant join().
	 */
	public void serverSendFileStop() throws InterruptedException {
		serviceStop(serverSendFileThread, "Interruption du service serveur send file");
	}


	/**
	 * Crée un serveur TCP qui reçoit du premier client connecté le contenu d'un fichier.
	 * @param port
	 *            port du serveur TCP.
	 * @param fileName
	 *            nom du fichier à écrire.
	 * @throws InterruptedException
	 *           exception en cas d'interruption pendant l'attente de la fin du thread.
	 */
	public void serverReceiveFileStart(int port, String fileName) throws InterruptedException {
		Runnable serverReceiveFileRun = new Runnable() {
			@Override
			public void run() {
				try (TcpServer sendFileServer = new TcpServer(port);
						FileOutputStream fout = new FileOutputStream(fileName);){
					if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
						TEST.trace("TcpTestTools: Lancement du service serveur receive file sur le port " + port + " et fichier " + fileName );
					}

					TcpSocket connectedToClient = sendFileServer.acceptClient();

					FileChannel fcout = fout.getChannel();
					ByteBuffer buffer = ByteBuffer.allocate(1024);
					int lu;
					do {
						buffer.clear();
						lu = connectedToClient.receiveBuffer(buffer);
						buffer.flip();
						fcout.write(buffer);
					} while (lu == 1024);
					fcout.close();
					fout.close();
					connectedToClient.close();
				} catch (InterruptedIOException e) {
					Thread.currentThread().interrupt();

				} catch (Exception e1) {
					if (e1.getMessage() != null) {
						if (e1.getMessage() != null) {
							if (LOG_ON && TEST.isEnabledFor(Level.DEBUG)) {
								TEST.debug("TcpTestTools:  Exception:" + e1.getMessage());
							}
						}
					}
					// fin du thread sans autre explication... TODO traiter les fins anormales!
				} 
				if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
					TEST.trace("TcpTestTools: Fin du service serveur receive file");
				}
				serverReceiveFileThread = null;
			}

		};
		serverReceiveFileThread = new Thread(serverReceiveFileRun);
		serverReceiveFileThread.start();
		Thread.sleep(1000); // pour être sûr que le serveur est lancé
	}

	/**
	 * Interrompt le thread qui réalise le service server receive file.
	 * @throws InterruptedException
	 *       exception en cas d'interruption pendant join().
	 */
	public void serverReceiveFileStop() throws InterruptedException {
		serviceStop(serverReceiveFileThread, "Interruption du service serveur receive file");
	}



	/**
	 * Crée un client TCP qui envoie au serveur localhost:port le contenu d'un fichier.
	 * @param port
	 *            port du serveur TCP.
	 * @param fileName
	 *            nom du fichier à envoyer.
	 */
	public void clientSendFileStart(int port, String fileName)  {
		Runnable clientSendFileRun = new Runnable() {
			@Override
			public void run() {
				try (TcpSocket client = new TcpSocket("localhost",port);
						FileInputStream fin = new FileInputStream(fileName);){
					if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
						TEST.trace("TcpTestTools: Lancement du service client send file sur le port " + port );
					}


					FileChannel fcin = fin.getChannel();
					ByteBuffer buffer = ByteBuffer.allocate(1024);
					int lu; 
					do {
						buffer.clear();
						lu = fcin.read(buffer);
						buffer.flip();
						client.sendBuffer(buffer);
					} while (lu == 1024);

				} catch (InterruptedIOException e) {
					Thread.currentThread().interrupt();

				} catch (Exception e1) {
					if (e1.getMessage() != null) {
						if (e1.getMessage() != null) {
							if (LOG_ON && TEST.isEnabledFor(Level.DEBUG)) {
								TEST.debug("TcpTestTools:  Exception:" + e1.getMessage());
							}
						}
					}
					// fin du thread sans autre explication... TODO traiter les fins anormales!
				} 
				if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
					TEST.trace("TcpTestTools: Fin du service client send file");
				}
				clientSendFileThread = null;

				

			}

		};
		clientSendFileThread = new Thread(clientSendFileRun);
		clientSendFileThread.start();
	}

	/**
	 * Interrompt le thread qui réalise le service client send file.
	 * @throws InterruptedException
	 *       exception en cas d'interruption pendant join().
	 */
	public void clientSendFileStop() throws InterruptedException {
		serviceStop(clientSendFileThread, "Interruption du service client send file");
	}



	/**
	 * Crée un client TCP qui reçoit du serveur localhost:port le contenu d'un fichier.
	 * @param port
	 *            port du serveur TCP.
	 * @param fileName
	 *            nom du fichier à écrire.
	 */
	public void clientReceiveFileStart(int port, String fileName) {
		Runnable clientReceiveFileRun = new Runnable() {
			@Override
			public void run() {
				try (TcpSocket client = new TcpSocket("localhost",port);
						FileOutputStream fout = new FileOutputStream(fileName);){
					if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
						TEST.trace("TcpTestTools: Lancement du service serveur receive file sur le port " + port + " et fichier " + fileName );
					}
					

					FileChannel fcout = fout.getChannel();
					ByteBuffer buffer = ByteBuffer.allocate(1024);
					int lu; 
					do {
						buffer.clear();
						lu = client.receiveBuffer(buffer);
						buffer.flip();
						fcout.write(buffer);
					} while (lu == 1024);
					fcout.close();
					fout.close();
					client.close();
				} catch (InterruptedIOException e) {
					Thread.currentThread().interrupt();

				} catch (Exception e1) {
					if (e1.getMessage() != null) {
						if (e1.getMessage() != null) {
							if (LOG_ON && TEST.isEnabledFor(Level.DEBUG)) {
								TEST.debug("TcpTestTools:  Exception:" + e1.getMessage());
							}
						}
					}
					// fin du thread sans autre explication... TODO traiter les fins anormales!
				} 

				if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
					TEST.trace("TcpTestTools: Fin du service client receive file");
				}
				clientReceiveFileThread = null;

				
			}

		};
		clientReceiveFileThread = new Thread(clientReceiveFileRun);
		clientReceiveFileThread.start();
	}

	/**
	 * Interrompt le thread qui réalise le service client recieve file.
	 * @throws InterruptedException
	 *       exception en cas d'interruption pendant join().
	 */
	public void clientReceiveFileStop() throws InterruptedException {
		serviceStop(clientReceiveFileThread, "Interruption du service client receive file");
	}

	
	
	/**
	 * Débute une alarme qui enverra une intérruption au thread courrant dans timeout milisecondes.
	 * Utiliser alarmStop() pour arrêter cette alarme.
	 * @param timeout
	 *           durée en millisecondes avant d'envoyer l'interruption.
	 */
	public void alarmStart(long timeout ) {
		Thread t = Thread.currentThread();
		
		Runnable alarmRun = new Runnable() {
			@Override
			public void run() {
				try {
					if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
						TEST.trace("TcpTestTools: Lancement du service d'alarme pour une durée de " + timeout );
					}
					
					Thread.sleep(timeout);
					t.interrupt();

				} catch (Exception e1) {
					if (e1.getMessage() != null) {
						if (e1.getMessage() != null) {
							if (LOG_ON && TEST.isEnabledFor(Level.DEBUG)) {
								TEST.debug("TcpTestTools:  Exception:" + e1.getMessage());
							}
						}
					}
					// fin du thread sans autre explication... TODO traiter les fins anormales!
				} 

				if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
					TEST.trace("TcpTestTools: Fin du service client send file");
				}
				alarmThread = null;


			}

		};
		alarmThread = new Thread(alarmRun);
		alarmThread.start();
	}

	/**
	 * Interrompt le thread qui réalise le service alarm.
	 * @throws InterruptedException
	 *       exception en cas d'interruption pendant join().
	 */
	public void alarmStop() throws InterruptedException {
		serviceStop(alarmThread, "Interruption du service d'alarme");
	}
}
