package tp4509.graphique;

import java.io.IOException;

import javax.swing.JFrame;


import tp4509.picross.PicrossApi;
import tp4509.tcp.ProtocoleException;

/**
 * Une tâche envoyée au Thread de Swing.
 * Crée la fenêtre d'affichage et la rend visible
 * 
 * @author Éric Lallet.
 */
public class RunnableSwing implements Runnable {
	
	/**
	 * Référence sur la fenètre de jeu. 
	 */
	private WindowSwing windowSwing;
	
	/**
	 * Reférence sur l'api du Picross.
	 */
	private PicrossApi picrossApi;
	
	/**
	 * Constructeur qui initialise l'api du Picross.
	 * @param picrossInfo
	 *         Reférence sur l'api du Picross.
	 */
	public RunnableSwing(final PicrossApi picrossInfo) {
		this.picrossApi = picrossInfo;
	}

	// méthode appelée par Swing pour traiter cette tâche.
	@Override
	public void run() {
		JFrame frame = new JFrame("Picross");
		try {
			windowSwing = new WindowSwing(picrossApi);


			frame.add(windowSwing);
			frame.pack();
			frame.setVisible(true);
			frame.setResizable(false);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		} catch (IOException | ProtocoleException e) {
			e.printStackTrace();
		}
	}


	/**
	 * getter sur le JPanel windowSwing.
	 * @return
	 *         le JPanel windowSwing qui contient toute l'image.
	 */
	public WindowSwing getWindowSwing() {
		return windowSwing;
	}


}
