//CHECKSTYLE:OFF
package tp4509.tcp;

import static dm4509.common.Log.LOGGER_NAME_TEST;
import static dm4509.common.Log.LOGGER_NAME_COMM;
import static dm4509.common.Log.LOG_ON;
import static dm4509.common.Log.COMM;
import static dm4509.common.Log.TEST;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.FileOutputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.SocketChannel;

import org.apache.log4j.Level;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;


import dm4509.common.Log;
import dm4509.tcpservices.TcpTestTools;

public class TestConnexionAsynchrone {
	static private TcpTestTools tcpTestTools;
	
	@BeforeClass
	static public void lanceTestTools() throws InterruptedException {
		tcpTestTools = new TcpTestTools();
		Log.configureALogger(LOGGER_NAME_TEST, Level.TRACE);
		Log.configureALogger(LOGGER_NAME_COMM, Level.INFO);
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestConnexionAsynchrone] ----------- Début des tests unitaires pour la classe ConnexionAsynchrone -----------");
		}
	}
	@AfterClass
	static public void finTest() throws InterruptedException {
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestConnexionAsynchrone] ----------- Fin des tests unitaires pour la classe ConnexionAsynchrone -----------\n");
		}
	}
	@Ignore	
	@Test
	public void testConnexionAsynchrone() throws Exception {
		
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestConnexionAsynchrone::testConnexionAsynchrone] ->  test du constructeur");
		}
		
		tcpTestTools.serverStart(40001);
		InetSocketAddress rcvAddress = new InetSocketAddress(InetAddress.getByName("localhost"), 40001);
		SocketChannel rwChan =  SocketChannel.open(rcvAddress);
		
		ConnexionAsynchrone connexion = new ConnexionAsynchrone(rwChan);
		
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestConnexionAsynchrone::testConnexionAsynchrone] -> le canal est construit? " + connexion.getRwChan());

		}
		assertTrue("Le canal est construit.", connexion.getRwChan() != null);
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestConnexionAsynchrone::testConnexionAsynchrone] -> le canal est non bloquant? " + !rwChan.isBlocking());

		}
		assertFalse("La connexion est asynchrone.", rwChan.isBlocking());
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestConnexionAsynchrone::testConnexionAsynchrone] -> le buffer est alloué? " + connexion.getBuffer());

		}
		assertTrue("Le buffer  est alloué.", connexion.getBuffer() != null);
		
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestConnexionAsynchrone::testConnexionAsynchrone] -> le buufer a la bonne taille? " + connexion.getBuffer().capacity());

		}
		assertTrue("Le buffer est dimmensionné pour le protocole.", connexion.getBuffer().capacity() == MessageProtocole.BUFFERSIZE);
		
		tcpTestTools.serverStop();
		connexion.close();
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestConnexionAsynchrone::testConnexionAsynchrone] ->  Fin du test ");
		}
	}

	@Ignore	
	@Test
	public void testAvancerLecture() throws Exception {
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestConnexionAsynchrone::testAvancerLecture] ->  test de lecture");
		}
		
		
		// lancement d'un serveur de test TCP qui va émettre les 2 derniers entiers
		// d'un message du protocole.
		FileOutputStream fout = new FileOutputStream("./tmp/deuxEntiers.data");
		FileChannel fcout = fout.getChannel();
		ByteBuffer buffer1 = ByteBuffer.allocate(2 * Integer.SIZE / Byte.SIZE);
		buffer1.putInt(2);
		buffer1.putInt(3);
		buffer1.flip();
		fcout.write(buffer1);
		fcout.close();
		fout.close();
		tcpTestTools.serverSendFileStart(40001,"./tmp/deuxEntiers.data");
		

		
		// réception des 2 derniers entiers avec la méthode avancerLecture().
		InetSocketAddress rcvAddress = new InetSocketAddress(InetAddress.getByName("localhost"), 40001);
		SocketChannel rwChan =  SocketChannel.open(rcvAddress);
		
		ConnexionAsynchrone connexion = new ConnexionAsynchrone(rwChan);
		
		// on simule une lecture précédente des 2 premiers entiers
		// d'un message du protocole.
		ByteBuffer	buffer = connexion.getBuffer();
		buffer.putInt(Protocole.INDICE_COLONNE.ordinal());
		buffer.putInt(1);
		
		// on lance la lecture de la fin du message.
		connexion.avancerLecture();
		
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestConnexionAsynchrone::testAvancerLecture] -> statut messageComplet ok? " + connexion.messageComplet() );

		}
		assertTrue("Le message est complet.",  connexion.messageComplet());
		ByteBuffer buffer2 = connexion.getBuffer();
		buffer2.rewind();
		MessageProtocole messageRecu = new MessageProtocole(buffer2);
		MessageProtocole messageEmis = new MessageProtocole(Protocole.INDICE_COLONNE, 1, 2, 3);
		
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestConnexionAsynchrone::testAvancerLecture] -> message reçu = message émis? " + messageRecu.equals(messageEmis)  );

		}
		assertTrue("Le message reçu = le message émis", messageRecu.equals(messageEmis));
		
		tcpTestTools.serverReceiveFileStop();
		connexion.close();
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestConnexionAsynchrone::testAvancerLecture] -> Fin du test");
		}
	}
	
	
	//                        !!!!!  ATTENTION   !!!!
	// le test ci dessous contient une boucle. Si votre méthode est mal écrite
	// cette boucle peut devenir une boucle infinie et le test ne se terminera jamais.
	@Ignore
	@Test
	public void testGetMessage() throws Exception {
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestConnexionAsynchrone::testGetMessage] ->  test de la réception du message");
		}
		
		// création d'un serveur qui envoie un message du protocole.
		MessageProtocole messageEmis = new MessageProtocole(Protocole.TAILLE_GRILLE, 10, 15, 0);
		FileOutputStream fout = new FileOutputStream("./tmp/message.data");
		FileChannel fcout = fout.getChannel();
		fcout.write(messageEmis.getBuffer());
		fcout.close();
		fout.close();
		tcpTestTools.serverSendFileStart(40001,"./tmp/message.data");
		
		InetSocketAddress rcvAddress = new InetSocketAddress(InetAddress.getByName("localhost"), 40001);
		SocketChannel rwChan =  SocketChannel.open(rcvAddress);
		
		ConnexionAsynchrone connexion = new ConnexionAsynchrone(rwChan);
		
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestConnexionAsynchrone::testGetMessage] -> getMessage retourne null qd pas complet? " +  connexion.getMessage());

		}
		assertTrue("Le message n'est pas disponible avant réception", connexion.getMessage() == null);
		
		
		//                        !!!!!  ATTENTION   !!!!
		// boucle potentiellement infinie si votre méthode est mal écrite.
		while (!connexion.messageComplet()) {
			connexion.avancerLecture();
		}
		
		MessageProtocole messageRecu = connexion.getMessage();
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestConnexionAsynchrone::testGetMessage] -> message retourné = message émis? " +  messageRecu.equals(messageEmis));

		}
		assertTrue("Le message reçu = le message émis" , messageRecu.equals(messageEmis) );
		
		
		// test un second appel pour le même message
		messageRecu = connexion.getMessage();
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestConnexionAsynchrone::testGetMessage] -> on peut appeller plusieurs fois getMessage()? " +  messageRecu.equals(messageEmis));

		}
		assertTrue("getMessage peut être appelé plusieurs fois." , messageRecu.equals(messageEmis) );
	
		connexion.close();
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestConnexionAsynchrone::testGetMessage] ->  Fin du test");
		}
		
	}

}
